﻿/*********************************************************************************************************
 *
 * Diese Klasse stellt den Prozess dar, der die Befehle für einen Server ausgeführt.
 * 
 * Von dieser Klasse werden soviele Objekte erstellt, wie Server in der server.txt vohanden sind.
 * Jedes Objekt dieser Klasse wird in einem eigenen Thread ausgeführt und läuft quasi parallel.
 * 
**********************************************************************************************************/
using System;
using System.IO;
using System.Text;
using System.Threading;
using System.Diagnostics;

namespace FastCmd
{
    class NewProzess
    {
        // Delegat für die Ereignisfunktion
        public delegate void ProzessCompletedDelegat(object sender, prozessReturnArgs e);
        
        // Funktion die das Ereignis darstellt
        public event ProzessCompletedDelegat prozessCompleted;

        // Ojekt zum ausführen eines Prozesses erstellen
        private Process myProzess = new Process();

        // nimmt die Variable "arguments" aus dem Konstruktor auf
        private string arguments = String.Empty;

        // nimmt die Variable "fileName" aus dem Konstruktor auf
        private string fileName = String.Empty;

        // nimmt die Variable "ping" aus dem Konstruktor auf
        private bool isPing = false;

        // nimmt die Variable "timeout" aus dem Konstruktor auf
        private int timeout = 0;

        // nimmt den Stream von stdOutput auf
        private StringBuilder stdOutput = null;

        // Anzahl der durch StdOutput empfangenden Zeilen
        private int numOutputLines = 0;

        // zeigt an ob der Prozess fehlerhaft beendet wurde
        private string taskError = String.Empty;

        // enthält die Fehlermeldung
        private string msgError = String.Empty;

        // Objekt zur Aufnahme der Rückgabewerte (taskError, msgError)
        prozessReturnArgs args = new prozessReturnArgs();

        // Konstruktor für die Klasse mit Übergabe der Parameter
        public NewProzess(string fileName, string arguments, bool isFile, bool isPing, int timeout)
        {
            try
            {
                // Argumente an die globale Variable übergeben
                this.arguments = arguments;

                // Argumente an die globale Variable übergeben
                this.fileName = fileName;

                // Argumente an die globale Variable übergeben
                this.isPing = isPing;

                // Argumente an die globale Variable übergeben
                this.timeout = timeout;
                
                // Ereignisfunktion für StdOutput festlegen
                myProzess.OutputDataReceived += new DataReceivedEventHandler(myProzess_OutputDataReceived);

                // Ereignisfunktion für StdError festlegen
                myProzess.ErrorDataReceived += new DataReceivedEventHandler(myProzess_ErrorDataReceived);

                // Ist der Parameter "arguments" eine Datei
                if (isFile == true)
                {
                    // Programmname
                    myProzess.StartInfo.FileName = fileName;

                    // Programmargumente
                    myProzess.StartInfo.Arguments = arguments;
                }
                // Ist der Parameter "arguments" ein Befehl
                else
                {
                    // Programmname
                    myProzess.StartInfo.FileName = "cmd.exe";

                    // ist %1 in arguments enthalten
                    if (arguments.Contains("%1"))
                    {
                        // Der Platzhalter wird durch den Servernamen ersetzt
                        fileName = fileName.Replace("%1", arguments);
                    }

                    // Damit das Programm an Ende automatisch beendet wird
                    fileName = string.Format(" /c {0}", fileName);

                    // Programmargumente
                    myProzess.StartInfo.Arguments = fileName;

                }

                // Prozess ist nicht sichtbar 
                myProzess.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;

                // StdOutput und StdError empfangen
                myProzess.StartInfo.UseShellExecute = false;

                // StdOutput zulassen
                myProzess.StartInfo.RedirectStandardOutput = true;

                // StdError zulassen
                myProzess.StartInfo.RedirectStandardError = true;

            }

            // Fehlerbehandlung
            catch (Exception ex)
            {
                Console.WriteLine("");
                Console.WriteLine("Prozess: " + fileName + " Argumente: " + arguments);
                Console.WriteLine("");
                Console.WriteLine(ex.Message);
                Console.WriteLine("");
            }

        }

        // Wird ausgelöst, wenn Daten über StdError eingehen
        void myProzess_ErrorDataReceived(object sendingProzess, DataReceivedEventArgs e)
        {
            try
            {
                // wenn Daten von StdError verfügbar sind
                if (!String.IsNullOrEmpty(e.Data))
                {
                    // Zeile hochzählen, wenn Daten verfügbar sind
                    numOutputLines++;

                    // Text dem StringBuilder hinzufügen
                    stdOutput.Append(Environment.NewLine + "ERROR: " + e.Data);

                    // Festlegen, das ein Fehler aufgetreten ist
                    taskError = arguments;
                }

            }

            // Fehlerbehandlung
            catch (Exception ex)
            {
                Console.WriteLine("");
                Console.WriteLine("Prozess: " + fileName + " Argumente: " + arguments);
                Console.WriteLine("");
                Console.WriteLine(ex.Message);
                Console.WriteLine("");
            }

        }

        // Wird ausgelöst, wenn Daten über StdOutput eingehen
        void myProzess_OutputDataReceived(object sendingProzess, DataReceivedEventArgs e)
        {
            try
            {
                // wenn Daten von StdOutput verfügbar sind
                if (!String.IsNullOrEmpty(e.Data))
                {
                    // Zeile hochzählen, wenn Daten verfügbar sind
                    numOutputLines++;

                    // Text dem StringBuilder hinzufügen
                    stdOutput.Append(Environment.NewLine + e.Data);
                }

            }

            // Fehlerbehandlung
            catch (Exception ex)
            {
                Console.WriteLine("");
                Console.WriteLine("Prozess: " + fileName + " Argumente: " + arguments);
                Console.WriteLine("");
                Console.WriteLine(ex.Message);
                Console.WriteLine("");
            }

        }

        // Funktion zum starten des Prozesses in dem Objekt
        public void StartProzess()
        {
            try
            {
                // Thread erstellen
                Thread myThread = new Thread(new ThreadStart(StartTask));

                // Thread starten
                myThread.Start();
            }

            // Die Ausnahme wird ausgelöst, wenn ein NULL-Verweis an die Methode übergeben wird.
            catch (System.ArgumentNullException ex)
            {
                Console.WriteLine("");
                Console.WriteLine("Prozess: " + fileName + " Argumente: " + arguments);
                Console.WriteLine("");
                Console.WriteLine(ex.Message);
                Console.WriteLine("");
            }

            // Die Ausnahme wird ausgelöst, wenn nicht genügend Arbeitsspeicher zur Verfügung steht.
            catch (System.OutOfMemoryException ex)
            {
                Console.WriteLine("");
                Console.WriteLine("Prozess: " + fileName + " Argumente: " + arguments);
                Console.WriteLine("");
                Console.WriteLine(ex.Message);
                Console.WriteLine("");
            }

            // Die Ausnahme wird bei allen anderen Fehlern ausgelöst.
            catch (Exception ex)
            {
                Console.WriteLine("");
                Console.WriteLine("Prozess: " + fileName + " Argumente: " + arguments);
                Console.WriteLine("");
                Console.WriteLine(ex.Message);
                Console.WriteLine("");
            }

        }

        // eigenen Task für den Prozess starten
        private void StartTask()
        {
            try
            {
                // Objekt erstellen
                stdOutput = new StringBuilder("");

                // wird ein Ping angefordert
                if (isPing == true)
                {
                    // Ping Objekt erstellen und ausführen
                    PingIP ping = new PingIP(arguments, timeout);

                    // Konnte ein Ping erfolgreich empfangen werden
                    if (ping.result == true)
                    {
                        // den Prozess starten
                        myProzess.Start();

                        // Startet asynchrones lesen des Outputstreams
                        myProzess.BeginOutputReadLine();

                        // Startet asynchrones lesen des Errorstreams
                        myProzess.BeginErrorReadLine();

                        // warten bis der Prozess beendet ist, maximal 10 Minuten
                        myProzess.WaitForExit(600000);

                        // Sind Daten von StdOutput vorhanden, dann in Textdatei schreiben
                        if (numOutputLines > 0)
                        {
                            // Dateiname für die StdOutput Rückmeldung festlegen
                            string place = string.Format("{0}\\tmp\\{1}.txt", Environment.CurrentDirectory, arguments);

                            // Objekt für den Stream von StdOutput erzeugen
                            StreamWriter file = new StreamWriter(place);

                            // Leerzeilen hinzufügen
                            stdOutput.Append(Environment.NewLine + Environment.NewLine);

                            // StdOutput in Textdatei schreiben
                            file.Write(stdOutput);

                            // Reste aus dem Puffer schreiben
                            file.Flush();

                            // Warten bis alles geschrieben ist
                            Thread.Sleep(50);

                            // StreamWriter schließen
                            file.Close();

                        }

                    }
                    // Ping nicht erfolgreich gewesen
                    else
                    {
                        taskError = arguments;
                        msgError = "Server nicht erreichbar !!!";
                    }

                }
                // Es wird kein Ping angefordert
                else
                {
                    // den Prozess starten
                    myProzess.Start();

                    // Startet asynchrones lesen des Outputstreams
                    myProzess.BeginOutputReadLine();

                    // Startet asynchrones lesen des Errorstreams
                    myProzess.BeginErrorReadLine();

                    // warten bis der Prozess beendet ist
                    myProzess.WaitForExit();

                    // Sind Daten von StdOutput vorhanden, dann in Textdatei schreiben
                    if (numOutputLines > 0)
                    {
                        // Dateiname für die StdOutput Rückmeldung festlegen
                        string place = string.Format("{0}\\tmp\\{1}.txt", Environment.CurrentDirectory, arguments);

                        // Objekt für den Stream von StdOutput erzeugen
                        StreamWriter file = new StreamWriter(place);

                        // Leerzeilen hinzufügen
                        stdOutput.Append(Environment.NewLine + Environment.NewLine);

                        // StdOutput in Textdatei schreiben
                        file.Write(stdOutput);

                        // Reste aus dem Puffer schreiben
                        file.Flush();

                        // Warten bis alles geschrieben ist
                        Thread.Sleep(50);

                        // StreamWriter schließen
                        file.Close();

                    }

                }

                // Status wird als Argument übergeben
                args.server = taskError;

                // Fehlermeldung wird als Argument übergeben
                args.ErrMsg = msgError;

                // Prozess beendet, Ereignis auslösen
                OnEvent(args);

            }

            // Fehlerbehandlung
            catch (Exception ex)
            {
                Console.WriteLine("");
                Console.WriteLine("Prozess: " + fileName + " Argumente: " + arguments);
                Console.WriteLine("");
                Console.WriteLine(ex.Message);
                Console.WriteLine("");
            }

        }

        // Funktion löst das Ereigis "prozessCompleted" aus
        protected virtual void OnEvent(prozessReturnArgs e)
        {
            try
            {
                // Ereignisfunktion für StdOutput lösen
                myProzess.OutputDataReceived -= new DataReceivedEventHandler(myProzess_OutputDataReceived);

                // Ereignisfunktion für StdError lösen
                myProzess.ErrorDataReceived -= new DataReceivedEventHandler(myProzess_ErrorDataReceived);

                // sind Empfänger für das Ereignis vorhanden
                if (prozessCompleted != null)

                    // Ereignis auslösen
                    prozessCompleted(this, e);
            }

            // Fehlerbehandlung
            catch (Exception ex)
            {
                Console.WriteLine("");
                Console.WriteLine("Prozess: " + fileName + " Argumente: " + arguments);
                Console.WriteLine("");
                Console.WriteLine(ex.Message);
                Console.WriteLine("");
            }

        }
        
    }

    // Klasse für die Rückgabeparameter
    public class prozessReturnArgs : EventArgs
    {
        public string server { get; set; }
        public string ErrMsg { get; set; }
    }

}
