﻿/*********************************************************************************************************
 *
 * Diese Klasse stellt ein Objekt dar, welches ein Ping auf den übergebenen Servernamen ausführt.
 * 
 * Quelle:
 * http://msdn.microsoft.com/en-us/library/system.net.networkinformation.ping%28v=vs.80%29.aspx
 * 
**********************************************************************************************************/
using System;
using System.Net;
using System.Text;
using System.Threading;
using System.Net.NetworkInformation;


namespace FastCmd
{
    class PingIP
    {
        // Ergebnis der Ping-Abfrage 
        bool status = false;

        // Eigenschaft der Klasse, enthält das Ergebnis aus der Ping-Abfrage
        public bool result
        {
            get { return status; }
        }

        // Konstruktor mit übergabe der Servers und timeout
        public PingIP(string hostName, int timeout)
        {
            try
            {
                IPHostEntry host;
                IPAddress Adresse = null;

                // Die IP-Adresse des Servers wird abgefragt
                host = Dns.GetHostEntry(hostName);

                // Liste aller IP-Adressen durchlaufen
                foreach (IPAddress ip in host.AddressList)
                    Adresse = ip;

                AutoResetEvent waiter = new AutoResetEvent(false);

                Ping pingSender = new Ping();

                // When the PingCompleted event is raised,
                // the PingCompletedCallback method is called.
                pingSender.PingCompleted += new PingCompletedEventHandler(PingCompletedCallback);

                // Create a buffer of 32 bytes of data to be transmitted.
                string data = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa";
                byte[] buffer = Encoding.ASCII.GetBytes(data);

                // Set options for transmission:
                // The data can go through 64 gateways or routers
                // before it is destroyed, and the data packet
                // cannot be fragmented.
                PingOptions options = new PingOptions(64, true);

                // Send the ping asynchronously.
                // Use the waiter as the user token.
                // When the callback completes, it can wake up this thread.
                pingSender.SendAsync(Adresse, timeout, buffer, options, waiter);

                // Prevent this example application from ending.
                // A real application should do something useful
                // when possible.
                waiter.WaitOne();
            }

            // Fehlerbehandlung
            catch (Exception)
            {
                // Ping nicht erfolgreich
                status = false;
            }

        }

        public void PingCompletedCallback(object sender, PingCompletedEventArgs e)
        {
            // If the operation was canceled or an error occurred, generate a message to the user.
            if ((e.Cancelled)||(e.Error != null))
            {
                // Ping nicht erfolgreich
                status = false;

                // Let the main thread resume. 
                // UserToken is the AutoResetEvent object that the main thread 
                // is waiting for.
                ((AutoResetEvent)e.UserState).Set();

                return;
            }

            // Ping erfolgreich
            status = true;

            // Let the main thread resume.
            ((AutoResetEvent)e.UserState).Set();

        }

    }

}
